// Simple invariant mass fit macro with correlation matrix 

// ROOT headers
#include "TStyle.h"
#include "TFile.h"
#include "TCanvas.h"
#include "TTree.h"
#include "TH1D.h"
#include "TH2D.h"
#include "TLegend.h"
#include "TRandom.h"

// RooFit headers
#include "RooGlobalFunc.h"
#include "RooRealVar.h"
#include "RooDataSet.h"
#include "RooDataHist.h"
#include "RooGaussian.h"
#include "RooArgList.h"
#include "RooFitResult.h"
#include "RooPlot.h"
#include "RooBinning.h"
#include "RooGenericPdf.h"
#include "RooCBShape.h"
#include "RooAddPdf.h"
using namespace RooFit ;
// --------------------------------------------------------------------------------

void mass_fit()
{

  // Binning
  RooBinning binM(100,2,5);
// --------------------------------------------------------------------------------
  // Cuts
  char strReduceM[120];
  sprintf(strReduceM,"jRecY>%f && jRecY<%f && jRecPt<%f && jRecM>%f && jRecM< %f",-4.0, -2.5,3., 2., 5.);
// --------------------------------------------------------------------------------
  // Create roofit variables

  RooRealVar jRecM("jRecM","jRecM",0,10) ;
  RooRealVar jRecPt("jRecPt","jRecPt",0,11) ;
  RooRealVar jRecY("jRecY","jRecY",-4,-2.5) ;

  jRecM.setBinning(binM);
  
//-------------------------------------------------------------------------------
  // Get data set

  TFile *f=new TFile("LHC15o_Train330.root"); 
  TTree *t;
  f->GetObject("jRecTree",t); 

  RooDataSet *dataIN = new RooDataSet("dataIN", "dataIN", RooArgSet(jRecM,jRecY,jRecPt), Import(*t));  // Read data from Tree
//------------------------------------------------------------------------------- 
  // Do cuts
  RooAbsData* dataM = dataIN->reduce(strReduceM);

//-------------------------------------------------------------------------------
  // Fit

  Int_t nEvts = dataM->numEntries();
  // Fit variables
    // Crystal Ball
  RooRealVar m0("m0","mass",3.096,3.05,3.15) ;
  RooRealVar n_cb("n_cb","n_cb",4.0,0,40) ;
  RooRealVar sig("sig","resolution",0.092,0.01,0.2) ;
  RooRealVar alpha("alpha","alpha",0.97,0,10) ;
    // Backkground
  RooRealVar lambda("lambda","exponent",-1.,-5.,0.) ;
  RooRealVar a1("a1","a1",0.002,0.0005,0.1) ;
  RooRealVar a2("a2","a2",0.015,0.005,0.1) ;
  RooRealVar a3("a3","a3",0.008,0.001,0.1) ;
  // Fit functions
  RooCBShape cb("cb","Crystal Ball PDF",jRecM,m0,sig,alpha,n_cb) ;
  RooGenericPdf Bkgd("Bkgd","exp(jRecM*lambda)*(1+a1*pow((jRecM-4),2)+a2*pow((jRecM-4),3)+a3*pow((jRecM-4),4))",RooArgSet(jRecM,lambda,a1,a2,a3));

  RooRealVar nCB("nCB","Number of CB events",0.5*nEvts,0,nEvts);
  RooRealVar nBG("nBG","Number of BG events",0.6*nEvts,0,nEvts);
  // Create Model
  RooAddPdf CbBG("CbBG","Crystal Ball plus Background PDF", RooArgList(Bkgd,cb), RooArgList(nBG,nCB));
  // Performe fit
  RooFitResult* rM = CbBG.fitTo(*dataM,Extended(kTRUE),Range(2.1,4.5),Save());

  // Compute number of J/Psi
  Double_t nCBM[2];
  jRecM.setRange("JPsiMassRange",0,10);
  RooAbsReal *iCB = cb.createIntegral(jRecM,NormSet(jRecM),Range("JPsiMassRange"));

  nCBM[0] = iCB->getVal()*nCB.getVal();
  nCBM[1] =  iCB->getVal()*nCB.getError();
 
 // -------------------------------------------------------------------------------- 
  // Plot results
    // Draw Correlation Matrix
  TCanvas *cPt = new TCanvas("cPt","cPt",900,600);
  cPt->cd(1);
  gStyle->SetOptTitle(0);
  gStyle->SetOptStat(0);
  gStyle->SetPalette(1);

  TH2* hcorr = rM->correlationHist();

   TPad *cPt_2 = new TPad("cPt_2", "cPt_2",0.01,0.01,0.99,0.99);
   cPt_2->Draw();
   cPt_2->cd();
   gPad->SetRightMargin(0.12);
   cPt_2->SetLeftMargin(0.1);
   gStyle->SetPaintTextFormat("4.2f");
   hcorr->SetMarkerSize(1.6);
   hcorr->GetXaxis()->SetLabelSize(0.049);
   hcorr->GetYaxis()->SetLabelSize(0.049);
   hcorr->Draw("colz,text");

    // Draw Histogram  
  TCanvas *cM = new TCanvas("cM","cM",900,600);
  cM->cd(1);
 
  TPad *cPt_1 = new TPad("cPt_1", "cPt_1",0.001,0.001,0.999,0.999);
  cPt_1->Draw();
  cPt_1->cd();
  cPt_1->SetRightMargin(0.05);

   
  RooPlot* frameM = jRecM.frame(Title("Mass fit")) ; 
  dataM->plotOn(frameM,Name("dataM"),Binning(binM),MarkerStyle(20),MarkerSize(1.));
  CbBG.plotOn(frameM,Name("CbBG"),LineColor(kBlack)) ;
  CbBG.plotOn(frameM,Name("cb"), Components(cb), LineColor(kRed));
  CbBG.plotOn(frameM,Name("Bkgd"), Components(Bkgd), LineColor(kBlue));
  frameM->GetYaxis()->SetLabelOffset(0.01);
  frameM->GetXaxis()->SetTitle("m (GeV)");
  frameM->Draw();

   // Get chi2 
   Double_t chi2M = frameM->chiSquare("CbBG","dataM",rM->floatParsFinal().getSize());

   // Legend
   TLegend *leg2 = new TLegend(0.6,0.6,0.9,0.9);
   leg2->SetHeader("Invariant Mass fit","C"); 
   leg2->AddEntry((TObject*)0,Form("Rapidity: %.1f<y<%.1f",-4.0, -2.5),"");   
   leg2->AddEntry("dataM","Data", "P");
   leg2->AddEntry("CbBG","Model","L");
   leg2->AddEntry("cb","Signal J/#psi","L");
   leg2->AddEntry("Bkgd","Background","L");
   leg2->AddEntry((TObject*)0,Form("#chi^{2}/NDF: %.3f",chi2M),"");
   leg2->AddEntry((TObject*)0,Form("N_{J/#psi} = %3.0f+/-%3.0f",nCBM[0],nCBM[1]),"");
   leg2->SetTextSize(0.027);
   leg2->Draw();

// save as pdf
cM->Print("Mass_fit.pdf");
cPt->Print("Mass_fit_corr_matrix.pdf");



	 
}
